/******************************************************************************
 * CoreWall / Corelyzer - An Initial Core Description Tool
 * Copyright (C) 2005 Arun Rao, Electronic Visualization Laboratory,  
 * University of Illinois at Chicago
 *
 * This software is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either Version 2.1 of the License, or 
 * (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public 
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser Public License along
 * with this software; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 *****************************************************************************/
#ifndef BASE_IMAGE_HANDLER_H
#define BASE_IMAGE_HANDLER_H

#ifdef _WIN32
#define DLL_EXPORT __declspec(dllexport)
#else
#define DLL_EXPORT
#endif

#define UNDEFINED_FORMAT 0

enum{
    RGB = 1,
    RGBA,
    BGR,
    BGRA,
    GRAY
};

enum{
    DPI = 1,    //Dots per inch
    DPM,    //Dots per m
    DPCM,    //Dots per cm
    DPMM    //Dots per mm
};

class ImageHandler{
public:
    DLL_EXPORT ImageHandler();
    DLL_EXPORT ImageHandler(const ImageHandler&, float scale = 1.0);

    DLL_EXPORT virtual ~ImageHandler();
    DLL_EXPORT virtual bool Load(char* filename);
    
    DLL_EXPORT char* GetPixels();
    DLL_EXPORT int GetPixelFormat();
    DLL_EXPORT int GetWidth();
    DLL_EXPORT int GetHeight();
    DLL_EXPORT float GetResolutionH();
    DLL_EXPORT float GetResolutionV();
    DLL_EXPORT int GetResolutionFormat();
    
    DLL_EXPORT void CloseImage();  // completely free the image!
    DLL_EXPORT char* GetSubImage(char* outbuf, int x, int y, int w, int h, 
        float scale );
    DLL_EXPORT void FreeSubImage(char* sub);
   
    DLL_EXPORT const char* GetSourceFilename();

    //methods to rotate images in 90 degree increments
    DLL_EXPORT virtual void Rotate90DegCW();
    DLL_EXPORT virtual void Rotate90DegCCW();
    DLL_EXPORT virtual void Rotate180Deg();
protected:    
    char* pixels;
    int width;
    int height;
    float resH;
	float resV;
    int pixelFormat;
    int resFormat;
    char* sourceFile; 
};


#endif
