/******************************************************************************
 * CoreWall / Corelyzer - An Initial Core Description Tool
 * Copyright (C) 2005 Arun Rao, Electronic Visualization Laboratory,  
 * University of Illinois at Chicago
 *
 * This software is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either Version 2.1 of the License, or 
 * (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public 
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser Public License along
 * with this software; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 *****************************************************************************/
#ifndef _WIN32
#include <unistd.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <png.h>
#include "pngimghandler.h"

#define DEBUG_PNG_LOAD 0

//-------------------------------------------

//-------------------------------------------
DLL_EXPORT PNGImageHandler::PNGImageHandler()
{
    ImageHandler::ImageHandler();
}

DLL_EXPORT PNGImageHandler::~PNGImageHandler()
{

}


DLL_EXPORT bool PNGImageHandler::Load( char* filename )
{
    if( filename == NULL )
        return false;
        
    if( !strstr(filename,".PNG") && !strstr(filename,".png") )
    {
        return false;
    }


    png_structp png_ptr;
    png_ptr = png_create_read_struct( PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
    if( !png_ptr )
        return false;

    png_infop info_ptr;
    info_ptr = png_create_info_struct(png_ptr);
    if( !info_ptr )
    {
        png_destroy_read_struct( &png_ptr, NULL, NULL );
        return false;
    }

    if( setjmp( png_jmpbuf( png_ptr )))
    {
        png_destroy_read_struct( &png_ptr, &info_ptr, NULL );
        return false;
    }

    FILE* fp = fopen( filename, "rb" );
    if(!fp)
    {
        fprintf(stderr,"Can't open file %s\n", filename);
        return false;
    }

    png_init_io(png_ptr,fp);
    png_read_png( png_ptr, info_ptr,
        PNG_TRANSFORM_STRIP_16 | PNG_TRANSFORM_PACKING, NULL );

    width = (int) png_get_image_width(png_ptr,info_ptr);
    height = (int) png_get_image_height(png_ptr,info_ptr);

    int channelCount;
    resFormat = DPM; //just assume DPM, as PNG doesn't have any other,
                    // just unknown otherwise
                   
    resH = png_get_x_pixels_per_meter(png_ptr,info_ptr);
    resV = png_get_y_pixels_per_meter(png_ptr,info_ptr);

    switch( png_get_color_type(png_ptr,info_ptr) )
    {
    case PNG_COLOR_TYPE_GRAY:
        pixelFormat = GRAY;
        channelCount = 1;
        break;
    case PNG_COLOR_TYPE_RGB:
        pixelFormat = RGB;
        channelCount = 3;
        break;
    case PNG_COLOR_TYPE_RGB_ALPHA:
        pixelFormat = RGBA;
        channelCount = 4;
        break;
    default:
        fprintf(stderr,"Unsupported color type!\n");
        png_destroy_read_struct( &png_ptr, &info_ptr, NULL );
        fclose(fp);
        return false;
    };
 
    //only support 8 bit
    if( info_ptr->bit_depth != 8 )
    {
       fprintf(stderr,"Unsupported bit depth!\n");
       png_destroy_read_struct( &png_ptr, &info_ptr, NULL);
       fclose(fp);
       return false;
    }

    png_bytep *b = png_get_rows( png_ptr, info_ptr );

    if( !b )
    {
        fprintf(stderr,"Can't get pixels from PNG\n");
        png_destroy_read_struct( &png_ptr, &info_ptr, NULL);
        fclose(fp);
        return false;
    }

    pixels = new char[ channelCount * width * height ];
    memset(pixels,0,channelCount * width * height );
    
    for( int y = 0; y < height; y++)
        for(int x = 0; x < width; x++)
            for( int i = 0; i < channelCount; i++)
                pixels[ y * width * channelCount + x * channelCount + i]
                    = (char) b[height - y - 1][x * channelCount + i];

    png_destroy_read_struct( &png_ptr, &info_ptr, NULL );
    fclose(fp);

    sourceFile = new char[ strlen(filename) + 1];
    strcpy( sourceFile, filename );

    return true;
}
